# pylint: disable=missing-module-docstring, missing-class-docstring, missing-function-docstring, not-callable

from os import PathLike
import requests_html

from akari_dl import src

class Website:
  def __init__(self, name=str, anime=str, output=PathLike, episodes=int, specials=bool, urls=str|tuple, anchors=tuple):

    self.name = name
    self.anime = anime
    self.output_path = output

    self.episodes_count = episodes
    self.specials_enabled = specials

    self.urls = urls # Available URLs to try to connect to requested site
    self.anchors = anchors # Expected anchor elements to scrape href from to go from the search page to the anime page to the video file link.

    self.url = str
    self.endpoint = None
    self.session  = requests_html.HTMLSession()

    # options.add_argument("User-Agent=Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Mobile Safari/537.36")


  def resolve_url(self):
    self.url = src.resolve_url(self)


  def resolve_anime(self):
    self.endpoint = src.resolve_anime(self)


  def download_anime(self):
    src.download_anime(self)



def log_response(response):
  """
    Log response headers and html into .log files (for development purpose).
  """
  with open("HEADERS.log", "w", encoding="utf-8") as header_dump:
    header_dump.write(str(response.headers))
  with open("BODY.log", "w", encoding="utf-8") as body_dump:
    body_dump.write(str(response.text))



"""
  Download an anime based on user-configuration given desired anime is found on desired website.
"""

import os

def download_episodes(self, anime_slug=str, episodes_regular=list):
  """
    Download all (unless specified otherwise) episodes of an anime
    into a folder of the anime's name inside the user-provided output path.
  """
  ep_count = 0

  folder_path = os.path.join(self.output_path, anime_slug)

  for episode in episodes_regular:
    ep_count += 1
    if self.name == "chauthanh":
      self.response = self.session.get(f"{self.url}/anime/{episode.attrs['href'][3:]}", timeout=30)
    else:
      self.response = self.session.get(f"{self.url}{episode.attrs['href']}", timeout=30)
    anchor = self.response.html.find(self.anchors[2], first=True)
    self.endpoint = anchor.attrs["href"]

    file_format = self.endpoint[-3:]

    if not os.path.exists(folder_path):
      os.makedirs(folder_path)

    if self.name == "chauthanh":
      self.response = self.session.get(f"{self.url}/anime/download/{self.endpoint[3:]}", timeout=30)
    else:
      self.response = self.session.get(self.endpoint, timeout=30)

    print(f"Downloading episode {ep_count} from {self.endpoint}")
    file_path = os.path.join(folder_path, f"Episode {ep_count}.{file_format}")
    with open(file_path, "wb") as video_file:
      for chunk in self.response.iter_content(1024):
        video_file.write(chunk)
    print(f"Episode {ep_count} downloaded to {file_path}")


def download_specials(self, anime_slug=str, episodes_special=list):
  """
    Download all (unless specified otherwise) episodes of an anime
    into a folder called "specials" inside a folder of the anime's name inside the user-provided output path.
  """
  ep_count = 0

  folder_path = os.path.join(self.output_path, anime_slug, "specials")

  for episode in episodes_special:
    ep_count += 1
    self.response = self.session.get(f"{self.url}{episode.attrs['href']}", timeout=30)
    anchor = self.response.html.find(self.anchors[2], first=True)
    self.endpoint = anchor.attrs["href"]

    file_format = self.endpoint[-3:]

    if not os.path.exists(folder_path):
      os.makedirs(folder_path)

    file_path = os.path.join(folder_path, f"Special {ep_count}.{file_format}")

    self.response = self.session.get(self.endpoint, timeout=30)

    print(f"Downloading special {ep_count} from {self.endpoint}")
    with open(file_path, "wb") as video_file:
      for chunk in self.response.iter_content(1024):
        video_file.write(chunk)
    print(f"Special {ep_count} downloaded to {file_path}.")


def download_anime(self):
  """
    Download user specified anime by scraping links until reaching a video file source.
  """
  self.response = self.session.get(f"{self.url}{self.endpoint}")
  episodes = self.response.html.find(self.anchors[1]) # Episodes anchors

  episodes_regular, episodes_special = [], []

  # Tokyoinsider puts the specials in the same list as the as the regular episodes.
  if self.name == "tokyoinsider":
    episodes.reverse()
    for episode in episodes:
      match episode.find("em", first=True).text:
        case "episode":
          episodes_regular.append(episode)
        case _:
          episodes_special.append(episode)
  else:
    episodes_regular = episodes

  anime_slug = self.anime
  for char in "/><\"\:|?*":
    anime_slug = anime_slug.replace(char, "")

  download_episodes(self, anime_slug, episodes_regular)

  if self.specials_enabled is True:
    download_specials(self, anime_slug, episodes_special)

  return f"Finished downloading {self.anime}."



def resolve_url(self):
  """
    Resolve working url for user-specified website.
  """
  print(f"Connecting to {self.name}...")

  if isinstance(self.urls, tuple):
    i = 0
    connected = False
    for url in self.urls:
      try:
        while not connected:
          i += 1
          self.response = self.session.get(f"https://{url}")
          if self.response.status_code == 200:
            connected = True
      except Exception:
        try:
          print(f"Failed to connect to {self.name} via https://{url}... trying https://{self.urls[i]}.")
        except IndexError:
          print(f"Failed to connect to {self.name}.")
          exit()
  else:
    try:
      self.response = self.session.get(f"https://{self.urls}")
    except Exception:
      print(f"Failed to connect to {self.name}.")
      exit()

  print(f"Connected to {self.response.url[:-1]}.")
  return self.response.url[:-1]



from setuptools import find_packages, setup

setup(
  name="akari-dl",
  version="1.0.0",
  author="keisan",
  author_email="<keisan@skiff.com>",
  description="A lightweight and open-source anime downloading CLI.",
  long_description="akari-dl downloads anime video files from direct download websites based on user configuration to avoid more annoying downloading methods like torrenting and manually downloading.",
  url="https://github.com/keisanng/akari-dl",
  project_urls={
    "Documentation": "https://github.com/keisanng/akari-dl#readme",
    "Tracker": "https://github.com/keisanng/projects/3"
  },
  packages=find_packages(),
  python_requires=">=3.10",
  install_requires=["requests_html"],
  entry_points={"console_scripts":["akari-dl=akari_dl.__main__:main"]}
)



"""
akari-dl
--------
A lightweight and open-source anime downloading CLI.

https://github.com/keisanng/akari-dl
"""

import logging
import argparse
from sys import version_info

if not version_info[0] >= 3 and not version_info[1] >= 10:
  print("You must use Python 3.10 or above.")
  exit()

parser = argparse.ArgumentParser(prog="akari_dl")

parser.add_argument("website", type=str, help="Specify the name of what website to direct-download anime from (see supported websites: https://github.com/keisanng/akari-dl#supported-websites.)")
parser.add_argument("anime", type=str, help="Specify what anime to download by title (in Romaji {https://en.wikipedia.org/wiki/Romanization_of_Japanese}.)")
parser.add_argument("output", type=str, help="Specify path to output downloaded video files to.")
parser.add_argument("-e", "--episodes", type=int, help="Specify the amount of episodes to download (downloads all if not specified) [NOT YET IMPLEMENTED.]")
parser.add_argument("-s", "--specials", action="store_true", help="Enable downloading of special episodes.")
parser.add_argument("-d", "--debug", action="store_true", help="Enable logging module for debugging.")
parser.add_argument("-v", "--version", action="version", version="1.0.0", help="Print the current version of akari-dl.")

logger = logging



from . import parser, logger
from .src.website import Website

def main():
  args = parser.parse_args()

  if args.debug is True:
    logger.basicConfig(level=logger.NOTSET)

  website = Website(args.website, args.anime, args.output, args.episodes, args.specials, urls=None, anchors=None)

  match args.website.lower():
    case "tokyoinsider":
      website.urls = ("tokyoinsider.com", "tokyoinsider.net", "tokyoinsider.org")
      website.anchors = ("table > tr > .c_h2 > a", ".episode > div > a", ".c_h2 > div > a")
    case "chauthanh":
      website.urls = "chauthanh.info"
      website.anchors = (".boxcontent > p > span > a", "tbody > tr > td > a", ".bd-blue > .p100 > a")
    case _:
      print(f"\033[91m\"{args.website}\" not supported.\033[0m")

  # pylint: disable=no-member
  website.resolve_url()
  website.resolve_anime()
  website.download_anime()

  print("Downloading Complete.")

if __name__ == "__main__":
  main()



def resolve_anime(self):
  """
    Verify user-specified anime exists on user-specified website via resolved url
    and return the link of the first entry of the anime's name on the website.
  """
  print(f"Searching for \"{self.anime}\"...")

  match self.name.lower():
    case "tokyoinsider":
      self.response = self.session.get(f"{self.url}/anime/search?k={self.anime}", timeout=30)
    case "chauthanh":
      self.response = self.session.get(f"{self.url}/search/?s={self.anime}", timeout=30)

  try:
    anchor = self.response.html.find(self.anchors[0], first=True)
    print("Anime found.")
  except Exception:
    print("Anime not found.")
    exit()

  return anchor.attrs["href"]
